----------------------------------------------------------------------------------
-- POLYTECHNIQUE MONTREAL
-- ELE3311 - Systemes logiques programmables 
-- 
-- Module Name:    spi_slave
-- Description:    Peripherique SPI
--
-- Additional Comments: 
--   Le peripherique SPI ne doit supporter que les valeur CPOL / CPHA par defaut.
----------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;

entity spi_slave is
generic (
  SPI_CPOL              : integer := 0;
  SPI_CPHA              : integer := 0;
  DATA_WIDTH            : integer := 12
);
port (
  sclk_i                : in    std_logic;
  mosi_i                : in    std_logic;
  miso_o                : out   std_logic;
  ss_n_i                : in    std_logic;
  rx_data_o             : out   std_logic_vector(DATA_WIDTH-1 downto 0);
  rx_data_valid_o       : out   std_logic;
  tx_data_i             : in    std_logic_vector(DATA_WIDTH-1 downto 0);
  tx_data_rdy_o         : out   std_logic
);
end spi_slave;


architecture behavioral of spi_slave is

  signal sclk                 : std_logic;
  -- *** A completer ***
  -- Signaux interne
  signal r_counter 		 : integer;
  signal spi_mode		   : integer;
  signal rx_data_buf	 : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal tx_data_buf	 : std_logic_vector(DATA_WIDTH-1 downto 0);

  signal parallel_load    : std_logic;
  signal compteur_r       : std_logic_vector(4 downto 0);

begin
  -- *** A completer ***
  
	-- Régler le spi mode
	spi_mode <= 0 when (SPI_CPOL = 0 and SPI_CPHA = 0) else 
              1 when (SPI_CPOL = 0 and SPI_CPHA = 1) else 
              2 when (SPI_CPOL = 1 and SPI_CPHA = 1) else 
              3 when (SPI_CPOL = 1 and SPI_CPHA = 0);
	
	-- Régler la polarité de l'horloge en fonction du mode
	sclk <= sclk_i when (spi_mode = 1 or spi_mode = 3) else not sclk_i;

	-- Compteur interne
	COMPTEUR: process(ss_n_i, sclk)
	begin
  	-- Reset asynchrone ou Inactif
		if (ss_n_i = '1') or (r_counter = 12) then
			if (SPI_CPHA = 0) then
				r_counter <= 2;
			else
				r_counter <= 1;
			end if;
			
			-- Transmettre et reset le buffer
			rx_data_o <= rx_data_buf;
			-- store txbuf (reset where?)
			-- reset miso
			--miso_o <= 'Z';
			
		-- Interface active
		else
			if (rising_edge(sclk)) then
				r_counter <= r_counter + 1;
			end if;
		end if;
	end process;

	-- main process
	process(ss_n_i, sclk)
	begin
	  -- Chargement parallèle des données à transmettre dans le registre cyclique
		if (r_counter = 12) or (ss_n_i = '1') then
			tx_data_rdy_o <= '1';
			tx_data_buf <= tx_data_i;
		else
			tx_data_rdy_o <= '0';
		end if;
		
    -- Registre reçu en entier
		if (r_counter = 12) then
			rx_data_valid_o <= '1';
		else
			rx_data_valid_o <= '0';
		end if;
				
		-- Recevoir sur MOSI
		if ss_n_i = '0' then -- not in reset state (superfluous?)
			if(SPI_CPHA = 0) then
				if(r_counter /= 2 and falling_edge(sclk)) then
					rx_data_buf(DATA_WIDTH-1 downto 0) <= rx_data_buf(DATA_WIDTH-2 downto 0) & mosi_i; --shift mosi bit in
				end if;
			else
				if(r_counter /= 1 and falling_edge(sclk)) then
					rx_data_buf(DATA_WIDTH-1 downto 0) <= rx_data_buf(DATA_WIDTH-2 downto 0) & mosi_i; --shift mosi bit in
				end if;
			end if;
		else
  		rx_data_buf <= (others => '0');
		end if;
		
		-- Transmettre sur MISO
		if ss_n_i = '0' then -- not in reset state (superfluous?)
			if (r_counter /= DATA_WIDTH) and (rising_edge(sclk)) then
				tx_data_buf(DATA_WIDTH-1 downto 0) <= tx_data_buf(DATA_WIDTH-2 downto 0) & tx_data_buf(DATA_WIDTH-1); --shift tx data
			end if;
		end if;
		
		if(ss_n_i = '0' and rising_edge(sclk)) then
			miso_o <= tx_data_buf(DATA_WIDTH-1);
		end if;
	end process;
end behavioral;
